import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:logging/logging.dart';
import 'package:result_monad/result_monad.dart';

import '../friendica_client/paged_response.dart';
import '../models/exec_error.dart';

final _logger = Logger('NetworkUtils');

FutureResult<PagedResponse<String>, ExecError> getUrl(
  Uri url, {
  Map<String, String>? headers,
}) async {
  _logger.finer('GET: $url');
  try {
    final response = await http.get(
      url,
      headers: headers,
    );

    if (response.statusCode != 200) {
      return Result.error(ExecError(
          type: ErrorType.authentication,
          message: '${response.statusCode}: ${response.reasonPhrase}'));
    }
    return PagedResponse.fromLinkHeader(
      response.headers['link'],
      utf8.decode(response.bodyBytes),
    );
  } catch (e) {
    return Result.error(
        ExecError(type: ErrorType.localError, message: e.toString()));
  }
}

FutureResult<String, ExecError> postUrl(
  Uri url,
  Map<String, dynamic> body, {
  Map<String, String>? headers,
}) async {
  _logger.finer('POST: $url \n Body: $body');
  try {
    final response = await http.post(
      url,
      headers: headers,
      body: jsonEncode(body),
    );

    if (response.statusCode != 200) {
      return Result.error(ExecError(
          type: ErrorType.authentication,
          message: '${response.statusCode}: ${response.reasonPhrase}'));
    }
    return Result.ok(utf8.decode(response.bodyBytes));
  } catch (e) {
    return Result.error(
        ExecError(type: ErrorType.localError, message: e.toString()));
  }
}

FutureResult<String, ExecError> putUrl(
  Uri url,
  Map<String, dynamic> body, {
  Map<String, String>? headers,
}) async {
  _logger.finer('PUT: $url \n Body: $body');
  try {
    final response = await http.put(
      url,
      headers: headers,
      body: jsonEncode(body),
    );

    if (response.statusCode != 200) {
      return Result.error(ExecError(
          type: ErrorType.authentication,
          message: '${response.statusCode}: ${response.reasonPhrase}'));
    }
    return Result.ok(utf8.decode(response.bodyBytes));
  } catch (e) {
    return Result.error(
        ExecError(type: ErrorType.localError, message: e.toString()));
  }
}

FutureResult<String, ExecError> deleteUrl(
  Uri url,
  Map<String, dynamic> body, {
  Map<String, String>? headers,
}) async {
  _logger.finer('DELETE: $url');
  try {
    final response = await http.delete(
      url,
      headers: headers,
      body: jsonEncode(body),
    );

    if (response.statusCode != 200) {
      return Result.error(ExecError(
          type: ErrorType.authentication,
          message: '${response.statusCode}: ${response.reasonPhrase}'));
    }
    return Result.ok(utf8.decode(response.bodyBytes));
  } catch (e) {
    return Result.error(
        ExecError(type: ErrorType.localError, message: e.toString()));
  }
}
